package pomodoro

import (
	"git.smsvc.net/pomodoro/GoTomato/internal/broadcast"
	"git.smsvc.net/pomodoro/GoTomato/pkg/models"
	"github.com/gorilla/websocket"
	"sync"
)

var PomodoroConfig = models.GoTomatoTimerConfig{
	Work:       15 * 60,
	ShortBreak: 5 * 60,
	LongBreak:  10 * 60,
	Sessions:   4,
}

var pomodoroOngoing bool
var pomodoroPaused bool

var pomodoroResetChannel = make(chan bool, 1)
var pomodoroPauseChannel = make(chan bool, 1)
var pomodoroResumeChannel = make(chan bool, 1)

var mu sync.Mutex // to synchronize access to shared state

// RunPomodoro iterates the Pomodoro work/break sessions.
func RunPomodoro(clients map[*websocket.Conn]*models.Client) {
	mu.Lock()
	pomodoroOngoing = true
	pomodoroPaused = false
	mu.Unlock()

	for session := 1; session <= PomodoroConfig.Sessions; session++ {
		if !startTimer(clients, PomodoroConfig.Work, "Work", session) {
			break
		}
		if session == PomodoroConfig.Sessions {
			if !startTimer(clients, PomodoroConfig.LongBreak, "LongBreak", session) {
				break
			}
		} else {
			if !startTimer(clients, PomodoroConfig.ShortBreak, "ShortBreak", session) {
				break
			}
		}
	}

	mu.Lock()
	pomodoroOngoing = false
	mu.Unlock()
}

// ResetPomodoro resets the running Pomodoro timer.
func ResetPomodoro(clients map[*websocket.Conn]*models.Client) {
	mu.Lock()
	pomodoroOngoing = false // Reset the running state
	pomodoroPaused = false  // Reset the paused state
	mu.Unlock()

	// Broadcast the reset message to all clients
	broadcast.BroadcastMessage(clients, models.ServerMessage{
		Mode:       "none",
		Session:    0,
		MaxSession: 0,
		TimeLeft:   0,
	})

	// Send a reset signal to stop any running timers
	pomodoroResetChannel <- true
}

func PausePomodoro() {
	mu.Lock()
	pomodoroPaused = true
	mu.Unlock()

	pomodoroPauseChannel <- true
}

func ResumePomodoro() {
	mu.Lock()
	pomodoroPaused = false
	mu.Unlock()
	pomodoroResumeChannel <- true
}

func IsPomodoroOngoing() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return pomodoroOngoing
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return pomodoroPaused
}
